@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2015 - 2018,  Ron <ron@debian.org>
+//  Copyright 2015 - 2021,  Ron <ron@debian.org>
 
 #ifndef _BB_ALIGNED_RECAST_H
 #define _BB_ALIGNED_RECAST_H
@@ -59,9 +59,7 @@ namespace BitB
     // but that seems wrong because it can throw and calls stringprintf, and
     // empirically, if we declare these with the const attribute then the unit
     // tests fail ...  so squelch the warning.
-   #pragma GCC diagnostic push
-   #pragma GCC diagnostic ignored "-Wpragmas"
-   #pragma GCC diagnostic ignored "-Wsuggest-attribute=const"
+    EM_TRY_PUSH_DIAGNOSTIC_IGNORE("-Wsuggest-attribute=const")
 
     // Safe cast back to a type with increased alignment.
     //
@@ -92,7 +90,7 @@ namespace BitB
         return aligned_recast< T, alignment_of<T>::value >( p );
     }
 
-   #pragma GCC diagnostic pop
+    EM_POP_DIAGNOSTIC
 }
 
 #endif  // _BB_ALIGNED_RECAST_H
