@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2003 - 2017,  Ron <ron@debian.org>
+//  Copyright 2003 - 2021,  Ron <ron@debian.org>
 
 #ifndef _BB_EXCEPTIONS_H
 #define _BB_EXCEPTIONS_H
@@ -13,24 +13,6 @@
 #include <cerrno>
 
 
-#define BB_NORETURN __attribute__ ((noreturn))
-
-
-#if EM_PLATFORM_MSW
-
- // The format checking misfires badly in mingw-w64 4.9.2-21+15.4
- // and complains about format characters that are supported, so
- // just disable it there until that gets fixed.
- #define BB_PRINTF_FORMAT( fmt, arg1, ... )
-
-#else
-
- #define BB_PRINTF_FORMAT( fmt, arg1, ... )                \
-        __attribute__ ((format (__printf__,fmt,arg1), ##__VA_ARGS__))
-
-#endif
-
-
 // If the thread cancellation exceptions are not supported, provide our own
 // definition of the exception type.  The main trick here is that we can't
 // put it directly into namespace abi if that is really an alias to some
@@ -55,6 +37,7 @@
 namespace BitB
 {
 
+    BB_PRINTF_FORMAT(2,0)
     static inline int Vasprintf( char **strp, const char *format, va_list arglist )
     { //{{{
 
@@ -105,6 +88,7 @@ namespace BitB
             m_msg = msg;
         }
 
+        BB_PRINTF_FORMAT(2,0)
         void SetMessage( const char *format, va_list args ) throw()
         {
             char    *msg = NULL;
@@ -128,6 +112,11 @@ namespace BitB
             va_end( arglist );
         }
 
+        void AppendMessage( const std::string &msg ) throw()
+        {
+            m_msg.append( msg );
+        }
+
         const char *what() const throw()
         {
             return m_msg.empty() ? "Unspecified BitB::Exception" : m_msg.c_str();
@@ -183,28 +172,24 @@ namespace BitB
         SystemError( const char *format, ... ) throw()
             : m_errno( errno )
         {
-            va_list         arglist;
-            std::string     msg( format );
-
-            msg.append( ": " ).append( strerror(m_errno) );
-
+            va_list   arglist;
             va_start( arglist, format );
-            SetMessage( msg.c_str(), arglist );
+            SetMessage( format, arglist );
             va_end( arglist );
+
+            AppendMessage( std::string(": ") + strerror(m_errno) );
         }
 
         BB_PRINTF_FORMAT(3,4)
         SystemError( int code, const char *format, ... ) throw()
             : m_errno( code )
         {
-            va_list         arglist;
-            std::string     msg( format );
-
-            msg.append( ": " ).append( strerror(m_errno) );
-
+            va_list   arglist;
             va_start( arglist, format );
-            SetMessage( msg.c_str(), arglist );
+            SetMessage( format, arglist );
             va_end( arglist );
+
+            AppendMessage( std::string(": ") + strerror(m_errno) );
         }
 
 
