@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2003 - 2018,  Ron <ron@debian.org>
+//  Copyright 2003 - 2021,  Ron <ron@debian.org>
 //
 // You must include bit-babbler/impl/log.h exactly once in some translation unit
 // of any program using the Log() functions.
@@ -271,6 +271,11 @@ namespace BitB
 
 
 
+    // On Windows we use the gnu_printf checking here (instead of letting it assume
+    // printf checking should use ms_printf), because we already rely on other mingw
+    // extensions, and with _GNU_SOURCE defined then (on C++) mingw will enable
+    // __USE_MINGW_ANSI_STDIO to support the full set of POSIX format characters.
+
     BB_PRINTF_FORMAT(1,2)
     std::string stringprintf( const char *format, ... );
 
@@ -278,6 +283,7 @@ namespace BitB
     std::string vstringprintf( const char *format, va_list arglist );
 
 
+    BB_COLD
     void GetFutureTimespec( timespec &ts, unsigned ms );
 
     static inline timeval GetWallTimeval()
@@ -299,7 +305,11 @@ namespace BitB
 
     } //}}}
 
-    BB_STRFTIME_FORMAT(1,0)
+    // And we use the gnu_strftime checking here because otherwise it will warn
+    // about using %T and %F (which we do use, and which the msvcrt.dll does not
+    // implement), but timeprintf will convert them to equivalents it is ok with
+    // if the Windows implementation of strftime(3) is used.
+    BB_STRFTIME_FORMAT(1)
     std::string timeprintf( const char *format, const timeval &tv = GetWallTimeval() );
 
 
@@ -427,9 +437,7 @@ namespace BitB
         if( msg.size() && msg[msg.size() - 1] == '\n' )
             msg.erase( msg.size() - 1 );
 
-        msg.append(": ").append( strerror(errno) ).append(1,'\n');
-
-        Log<N>( "%s", msg.c_str() );
+        Log<N>( "%s: %s\n", msg.c_str(), strerror(errno) );
         va_end( arglist );
 
     } //}}}
@@ -447,9 +455,7 @@ namespace BitB
         if( msg.size() && msg[msg.size() - 1] == '\n' )
             msg.erase( msg.size() - 1 );
 
-        msg.append(": ").append( strerror(code) ).append(1,'\n');
-
-        Log<N>( "%s", msg.c_str() );
+        Log<N>( "%s: %s\n", msg.c_str(), strerror(code) );
         va_end( arglist );
 
     } //}}}
