@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2013 - 2015,  Ron <ron@debian.org>
+//  Copyright 2013 - 2021,  Ron <ron@debian.org>
 
 #ifndef _BB_MATH_H
 #define _BB_MATH_H
@@ -10,6 +10,8 @@
 namespace BitB
 {
 
+    // Implicit modulo 1<<32 is part of the algorithm, so tell UBSan to relax.
+    BB_NO_SANITIZE_UNSIGNED_INTEGER_OVERFLOW
     static inline uint32_t popcount( uint32_t v )
     {
         // This can be notoriously crappy, but we don't care a lot here
@@ -26,14 +28,26 @@ namespace BitB
         return v ? sizeof(unsigned) * 8 - unsigned(__builtin_clz(v)) : 0;
     }
 
-    static inline unsigned powof2_down( unsigned v )
+    static inline unsigned fls( unsigned long v )
     {
-        return 1u << (fls(v) - 1);
+        return v ? sizeof(unsigned long) * 8 - unsigned(__builtin_clzl(v)) : 0;
     }
 
-    static inline unsigned powof2_up( unsigned v )
+    static inline unsigned fls( unsigned long long v )
     {
-        return 1u << fls(v - 1);
+        return v ? sizeof(unsigned long long) * 8 - unsigned(__builtin_clzll(v)) : 0;
+    }
+
+    template< typename T >
+    BB_CONST T powof2_down( T v )
+    {
+        return T(1) << (fls(v) - 1);
+    }
+
+    template< typename T >
+    BB_CONST T powof2_up( T v )
+    {
+        return T(1) << fls(v - 1);
     }
 
 }
