@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2010 - 2018,  Ron <ron@debian.org>
+//  Copyright 2010 - 2021,  Ron <ron@debian.org>
 
 #ifndef _BB_SECRET_SOURCE_H
 #define _BB_SECRET_SOURCE_H
@@ -12,6 +12,10 @@
  #include <poll.h>
 #endif
 
+#if defined(__FreeBSD__) && __FreeBSD__ < 13
+ #include <pthread_np.h>
+#endif
+
 #if EM_PLATFORM_MAC
  #include <fcntl.h>
 #endif
@@ -1059,9 +1063,7 @@ namespace BitB
                 // Don't warn about enum values not being explicitly handled here,
                 // we don't want to have to chase every new error code added to
                 // libusb that we don't explicitly care about handling here.
-               #pragma GCC diagnostic push
-               #pragma GCC diagnostic ignored "-Wpragmas"
-               #pragma GCC diagnostic ignored "-Wswitch-enum"
+                EM_PUSH_DIAGNOSTIC_IGNORE("-Wswitch-enum")
 
                 switch( e.GetErrorCode() )
                 {
@@ -1082,7 +1084,7 @@ namespace BitB
                     default:
                         throw;
                 }
-               #pragma GCC diagnostic pop
+                EM_POP_DIAGNOSTIC
             }
 
         } //}}}
@@ -1353,7 +1355,7 @@ namespace BitB
                     lock.Unlock();
                     pthread_cancel( p );
 
-                   #ifdef __FreeBSD__
+                   #if defined(__FreeBSD__) && __FreeBSD__ < 13
                     //{{{
                     // On FreeBSD 11 (where hotplug support was first added)
                     // if the device was removed while we were in an active
@@ -1368,6 +1370,9 @@ namespace BitB
                     //
                     // It's not ideal, but it's about the best we can do until
                     // the FreeBSD side of things is fixed.
+                    //
+                    // Since it looks like FreeBSD 13 doesn't have this problem
+                    // anymore, it can use the same code as everyone else again.
 
                     Log<4>( "Pool::RemoveSource: cancelling thread for %s\n",
                                                     d->VerboseStr().c_str() );
@@ -1548,13 +1553,13 @@ namespace BitB
                 struct rand_pool_info   rpi;
             };
 
-            HealthMonitor   qa( "Pool" );
             uint8_t        *buf = reinterpret_cast<uint8_t*>( rpi.buf );
-
-            HealthMonitor   qa2( "Kernel" );
             uint8_t         b2[N];
             size_t          b2_fill = 0;
 
+            HealthMonitor   qa( "Pool" );
+            HealthMonitor   qa2( "Kernel" );
+
             bool            source_ok;
             bool            folded_ok = false;
 
@@ -1594,12 +1599,16 @@ namespace BitB
                 if( ioctl( fd, RNDADDENTROPY, &rpi ) )
                     throw SystemError( _("Pool::FeedKernelEntropy: ioctl failed") );
 
+                EM_TRY_PUSH_DIAGNOSTIC_IGNORE("-Wgnu-designator")
+
                 struct pollfd   p = { fd: fd, events: POLLOUT, revents: 0 };
                 int             r = poll( &p, 1, timeout );
 
                 if( r < 0 )
                     throw SystemError( _("Pool::FeedKernelEntropy: poll failed") );
 
+                EM_POP_DIAGNOSTIC
+
                #elif EM_PLATFORM_MAC
 
                 // MacOS has no method of signalling to us when its kernel might
@@ -1673,7 +1682,7 @@ namespace BitB
                 // explicitly assign a pool to a monitor that enumerated bare
                 // (or broken) devices ...  so, bark and then ignore them.
                 Log<0>( _("DevList::DeviceAdded: "
-                          "not adding device with no serial number to the pool: %s"),
+                          "not adding device with no serial number to the pool: %s\n"),
                                                             d->VerboseStr().c_str() );
                 return;
             }
@@ -1882,7 +1891,7 @@ namespace BitB
                 // This should only happen if we lose some race.
                 // Either the device was unplugged again before we got here,
                 // or libusb somehow doesn't know that it exists yet ...
-                Log<0>( _("UDEVMonitor: failed to find device %03u:%03u"), busnum, devnum );
+                Log<0>( _("UDEVMonitor: failed to find device %03u:%03u\n"), busnum, devnum );
                 return NULL;
             }
 
@@ -1934,7 +1943,7 @@ namespace BitB
                 d->SetDevicePort( devport );
 
             else if( devport != d->GetDevicePort() )
-                Log<0>( _("UDEVMonitor: udev says device has port '%s', but libusb reported '%s'"),
+                Log<0>( _("UDEVMonitor: udev says device has port '%s', but libusb reported '%s'\n"),
                                                     devport.c_str(), d->GetDevicePort().c_str() );
 
             d->SetDevpath( devpath );
@@ -2376,7 +2385,7 @@ namespace BitB
             if( libusb_has_capability(LIBUSB_CAP_HAS_HOTPLUG) )
            #endif
             {
-               #ifdef __FreeBSD__
+               #if defined(__FreeBSD__) && __FreeBSD__ < 13
                 // On FreeBSD 11, we're seeing a 4 second delay between when we
                 // register the hotplug callback, and when it actually gets called
                 // to enumerate any already available devices.  In "normal" use,
@@ -2390,6 +2399,10 @@ namespace BitB
                 //
                 // There does seem to be some internal issue there, since we are
                 // also seeing libusb_exit() block for 4 seconds at shutdown too.
+                //
+                // With FreeBSD 13 there is still a 4 second delay in libusb_exit
+                // but devices are enumerated immediately so we don't need to do
+                // this manually anymore.
                 EnumerateDevices( vendorid, productid );
                #endif
 
