@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2015 - 2016,  Ron <ron@debian.org>
+//  Copyright 2015 - 2018,  Ron <ron@debian.org>
 
 #ifndef _BB_SOCKET_SOURCE_H
 #define _BB_SOCKET_SOURCE_H
@@ -28,6 +28,8 @@ namespace BitB
         void do_server_thread()
         { //{{{
 
+            SetThreadName( "UDP out" );
+
             std::string     addr = m_sa.AddrStr();
 
             Log<3>( "SocketSource( %s ): begin server_thread\n", addr.c_str() );
@@ -76,13 +78,13 @@ namespace BitB
                    #endif
 
                     if( n == -1 )
-                        LogSocketErr<1>( _("SocketSource( %s ): sendto failed\n"), addr.c_str() );
+                        LogSocketErr<1>( _("SocketSource( %s ): sendto failed"), addr.c_str() );
                     else if( size_t(n) != r )
                         Log<2>( _("SocketSource( %s ): only %zd of %zu bytes sent\n"),
                                                                     addr.c_str(), n, r );
                 }
                 else if( n == -1 )
-                    LogSocketErr<1>( _("SocketSource( %s ): recvfrom failed\n"), addr.c_str() );
+                    LogSocketErr<1>( _("SocketSource( %s ): recvfrom failed"), addr.c_str() );
                 else
                     Log<2>( _("SocketSource( %s ): ignoring %zd byte message\n"),
                                                                 addr.c_str(), n );
@@ -128,7 +130,7 @@ namespace BitB
         typedef RefPtr< SocketSource >  Handle;
 
 
-        SocketSource( const Pool::Handle &pool, const std::string &addr )
+        SocketSource( const Pool::Handle &pool, const std::string &addr, bool freebind = false )
             : m_pool( pool )
             , m_sa( addr )
         { //{{{
@@ -147,19 +149,23 @@ namespace BitB
             if( m_fd == -1 )
                 throw SocketError( _("SocketSource( %s ): failed to open socket"),
                                                                     addr.c_str() );
+            try {
+                if( freebind )
+                    EnableFreebind( m_fd, stringprintf("SocketSource( %s )", addr.c_str()) );
 
-            if( bind( m_fd, &m_sa.addr.any, m_sa.addr_len ) == -1 )
-            {
-                Close();
-                throw SocketError( _("SocketSource( %s ): bind failed"), addr.c_str() );
+                if( bind( m_fd, &m_sa.addr.any, m_sa.addr_len ) == -1 )
+                    throw SocketError( _("SocketSource( %s ): bind failed"), addr.c_str() );
+
+                int ret = pthread_create( &m_serverthread, GetDefaultThreadAttr(), server_thread,
+                                                                                            this );
+                if( ret )
+                    throw SystemError( ret, _("SocketSource( %s ): failed to create server thread"),
+                                                                                    addr.c_str() );
             }
-
-            int ret = pthread_create( &m_serverthread, GetDefaultThreadAttr(), server_thread, this );
-            if( ret )
+            catch( ... )
             {
                 Close();
-                throw SystemError( ret, _("SocketSource( %s ): failed to create server thread"),
-                                                                                addr.c_str() );
+                throw;
             }
 
         } //}}}
