@@ -1642,7 +1642,6 @@ namespace BitB
 
             Log<2>( "- USBContext\n" );
 
-
             // Release all the device references before destroying the context.
             // The libusb_device doesn't keep a reference count for the context
             // but it does have a pointer to it which something might access.
@@ -1825,13 +1824,10 @@ namespace BitB
             std::string     msg;
 
             va_start( arglist, format );
-            msg.append( vstringprintf( format, arglist ) )
-               .append( ": " )
-               .append( libusb_strerror(libusb_error(err)) );
+            msg = vstringprintf( format, arglist );
+            Log<N>( "%s: %s\n", msg.c_str(), libusb_strerror(libusb_error(err)) );
             va_end( arglist );
 
-            Log<N>( "%s\n", msg.c_str() );
-
         } //}}}
 
     }; //}}}
