@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2004 - 2018,  Ron <ron@debian.org>
+//  Copyright 2004 - 2021,  Ron <ron@debian.org>
 
 #ifndef _BB_USERS_H
 #define _BB_USERS_H
@@ -53,7 +53,7 @@ namespace BitB
                     bufsize <<= 1;
                     goto try_again;
                 }
-                // fall through
+                BB_FALLTHROUGH; // fall through
 
             default:
                 delete [] buf;
